/*
   wmthemer, Window Maker theme changer
   Copyright (C) 2001-2002 Gianluca Montecchi <gianluca@pluto.linux.it>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <dockapp.h>
#include <unistd.h>
#include <time.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>

#include "wmthemer.xpm"

#define RANDOM		1000
#define NORANDOM	1001

char *displayName = "";
int Interval = 30;
int Random = NORANDOM;
int Min_Interval;
char *pid_file_path;
char *theme_used;
char *home_dir;
char *rc_file_path;

static DAProgramOption options[] = {
    {"-display", NULL, "display to use", DOString, False, {&displayName} },
};

struct THEME {
    char theme[255];
    struct THEME *next;
};

struct PATH {
   char directory[255];
   struct PATH *next;
};

int close_wmthemer(int signal_called) {

	free(home_dir);
	free(pid_file_path);
	free(rc_file_path);
	free(theme_used);
	unlink(pid_file_path);
	exit(1);

}

int main(int argc, char **argv) {
    Pixmap pixmap;
    Pixmap backdrop;
    GC gc;
    static unsigned d_width = 48;
    static unsigned d_height = 48;
    struct THEME *first_item;
    struct THEME *current_item;
    struct PATH *first_dir;
    struct PATH *current_dir;
    DIR *theme_dir;
    struct dirent *dir_entry;
    int number_of_theme = 0;
    FILE *rnd_file;
    FILE *rc_file;
    FILE *pid_file;
    int seme, shift_value, path_num=0,tmp_val;
    char rnd_seme;
    char dummy[32]="";
 		
    (char *)home_dir = (char *)malloc(255);
    (char *)pid_file_path = (char *)malloc(255);
    (char *)theme_used = (char *)malloc(512);
    (char *)rc_file_path = (char *)malloc(255);
	   
    DAParseArguments(argc, argv, options,
                     sizeof(options)/sizeof(DAProgramOption),
                     "WMThemer", "WMThemer v0.9.5");

    strncpy((char *)home_dir, getenv("HOME"), 255);
    sprintf(rc_file_path, "%s/%s", home_dir, ".wmthemerrc");
    sprintf(pid_file_path, "%s/%s", home_dir, ".wmthemer.pid");
    pid_file = fopen(pid_file_path, "r");
    if (pid_file == NULL) {
		pid_file = fopen(pid_file_path, "w");
		fclose(pid_file);
    } else {
		printf("Another session of WMTheme is running...\nExiting.\n");
		exit(1);
    }
    signal(SIGINT, (void *)close_wmthemer);
    signal(SIGTERM, (void *)close_wmthemer);
    signal(SIGHUP, (void *)close_wmthemer);
    signal(SIGPIPE, (void *)close_wmthemer);

    rc_file = fopen(rc_file_path, "r");
    if (rc_file == NULL) {
		rc_file = fopen(rc_file_path, "w");
		fprintf(rc_file, "interval = %d\n", -1);
		fprintf(rc_file, "min_interval = %d\n", 300);
		fprintf(rc_file, "path = %s\n", "/usr/local/share/WindowMaker/Themes");
		fclose(rc_file);
    }
    rc_file = fopen(rc_file_path, "r");
    fscanf(rc_file, "%s = %d", dummy, &tmp_val);
    if (strncmp(dummy, "interval", 32) == 0) {
    	Interval = tmp_val;
#ifdef _DEBUG_
    	printf("Interval %d\n", Interval);
#endif
    } else if(strncmp(dummy, "min_interval", 32) == 0) {
    	Min_Interval = tmp_val;
#ifdef _DEBUG_   	
    	printf("Min Interval %d\n", Min_Interval);
#endif   	
    } else {
    	printf("Aborting, error in the configuration file\n");
    	unlink(pid_file_path);
    	exit(1);
	}
	
    fscanf(rc_file, "%s = %d", dummy, &tmp_val);
    if (strncmp(dummy, "interval", 32) == 0) {
    	Interval = tmp_val;
#ifdef _DEBUG_
    	printf("Interval %d\n", Interval);
#endif
    } else if(strncmp(dummy, "min_interval", 32) == 0) {
    	Min_Interval = tmp_val;
#ifdef _DEBUG_   	
    	printf("Min Interval %d\n", Min_Interval);
#endif   	
    } else {
    	printf("Aborting, error in the configuration file\n");
    	unlink(pid_file_path);
    	exit(1);
	}

    if (Interval == -1)
       Random = RANDOM;
    if (rc_file != NULL) {
		first_dir = (struct PATH *)malloc(sizeof(struct PATH));
		if(first_dir == NULL) {
			printf("Malloc failed allocating struct for path...\nExiting");
			exit(1);
		}
		current_dir = first_dir;
 	   while(!feof(rc_file)) {
           fscanf(rc_file, "%s = %s\n",dummy, home_dir);
           if (strncmp(dummy, "path",32) == 0) {
	           strncat(current_dir->directory, home_dir, 255);
 	          current_dir->next = (struct PATH *)malloc(sizeof(struct PATH));
  	         current_dir = current_dir->next;
   	        path_num++;
   	    } else {
   	    	printf("Aborting, error in the configuration file\n");
   	    	printf("One or more path specification contains error(s)\n");
		   	unlink(pid_file_path);
  		 	exit(1);
  		 }
        }
        current_dir->next = NULL;
    }
    fclose(rc_file);
#ifdef _DEBUG_
    printf("Number of paths %d\n", path_num);
#endif
    DAInitialize(displayName, "WMTheme", d_width, d_height, argc, argv);

    DAMakePixmapFromData(wmthemer_xpm, &backdrop, None, &d_width, &d_height);
    pixmap = DAMakePixmap();
    gc = DefaultGC(DADisplay, DefaultScreen(DADisplay));

    DAShow();
    /*
     * FIXME
     * I need to make use also the nested directory for the theme
     * but I am not sure if a theme can have nested directory
    */
    first_item = (struct THEME *)malloc(sizeof(struct THEME));
    if(first_item == NULL) {
        printf("Malloc for first item failed...\nExiting");
        exit(1);
    }
    current_item = (struct THEME *)malloc(sizeof(struct THEME));
    if(current_item == NULL) {
        printf("Malloc for n-type item failed...\nExiting");
        exit(1);
    }
    current_item = first_item; 

    current_dir = first_dir;
    while(current_dir->next != NULL) {
	    theme_dir = opendir(current_dir->directory);
 	   if (theme_dir == NULL) {
  	      printf("Error opening theme directory\n"),
  	      unlink(pid_file_path);
   	     exit(1);
    	}
	    dir_entry = readdir(theme_dir);
 	   dir_entry = readdir(theme_dir);
  	  dir_entry = readdir(theme_dir);
   	 if(dir_entry == NULL)
    	   break;
	    number_of_theme++;
 	   strncpy(first_item->theme, current_dir->directory, 255);
  	  strncat(first_item->theme, dir_entry->d_name, 255);
   	 first_item->next = (struct THEME*)malloc(sizeof(struct THEME));
    	while(1) {
	        dir_entry = readdir(theme_dir);
 	       if (dir_entry != NULL) {
  	          sprintf(current_item->theme, "%s/%s", current_dir->directory, dir_entry->d_name);
   	         number_of_theme++;
    	        current_item->next = (struct THEME *)malloc(sizeof(struct THEME));
     	       current_item = current_item->next;
      	  } else {
       	     break;
        	}
	    }
 	   closedir(theme_dir);
  	  current_dir = current_dir->next;
	}
    current_item->next = first_item;
    rnd_file = fopen("/dev/urandom", "r");
    if (rnd_file == NULL) {
        srandom(0);
    } else {
        fscanf(rnd_file,"%c", &rnd_seme);
        seme = (long)rnd_seme;
        if (seme <= 0)
            seme = seme * -1;
        srandom(seme);
    }
    while (1) {
		XEvent ev;
		XCopyArea(DADisplay, backdrop, pixmap, gc, 0, 0, 48, 48, 0, 0);
        seme = random() % number_of_theme;
        if (seme == 0)
            seme++;
        if (Random == RANDOM) {
           Interval = random() % 20 + Min_Interval;
        }
        
        for(shift_value = 0; shift_value <= seme; shift_value++) {
            current_item = current_item->next;
        }
        /*
         * FIXME
         * I need to find why there is an empty item in the list.
         * For now the work-around works, this is a minor problem
        */
        if(strcmp(current_item->theme, "") == 0) {
            current_item = current_item->next;
        }
		strcpy((char *)theme_used, "setstyle ");
        strcat((char *)theme_used, current_item->theme);
        
        DASetPixmap(pixmap);

		system((char *)theme_used);
		if (Min_Interval == -1) {
			break;
		}
		sleep(Interval);
		/* handle all pending X events */
		while (XPending(DADisplay)) {
		    XNextEvent(DADisplay, &ev);
	 	   DAProcessEvent(&ev);
		}
    }

    free(home_dir);
    free(pid_file_path);
    free(rc_file_path);
    free(theme_used);
    unlink(pid_file_path);
    return 0;
}


